@@ -1,3 +1,4 @@
+import abc
 import csv
 import functools
 import logging
@@ -112,15 +113,11 @@ class FileStore(object):
             f.close()
 
 
-class Operation(object):
+class Operation(metaclass=abc.ABCMeta):
     def __init__(self, connection):
         self.steps = []
         self.connection = connection
         self._bulk = None
-        self.describe_info = {}
-        self.field_maps = {}
-        self.proxy_objects = {}
-        self.key_prefix_map = None
         self.logger = logging.getLogger("amaxa")
         self.file_store = FileStore()
 
@@ -138,6 +135,7 @@ class Operation(object):
         for s in self.steps:
             s.initialize()
 
+    @abc.abstractmethod
     def execute(self):
         pass
 
@@ -149,30 +147,13 @@ class Operation(object):
         return [step.sobjectname for step in self.steps]
 
     def get_sobject_name_for_id(self, id):
-        if self.key_prefix_map is None:
-            global_describe = self.connection.get_global_describe()["sobjects"]
-            self.key_prefix_map = {
-                sobject["keyPrefix"]: sobject["name"] for sobject in global_describe
-            }
-
-        return self.key_prefix_map[id[:3]]
+        return self.connection.get_sobject_name_for_id(id)
 
     def get_describe(self, sobjectname):
-        if sobjectname not in self.describe_info:
-            self.describe_info[sobjectname] = self.connection.get_sobject_describe(
-                sobjectname
-            )
-            self.field_maps[sobjectname] = {
-                f.get("name"): f for f in self.describe_info[sobjectname].get("fields")
-            }
-
-        return self.describe_info[sobjectname]
+        return self.connection.get_sobject_describe(sobjectname)
 
     def get_field_map(self, sobjectname):
-        if sobjectname not in self.describe_info:
-            self.get_describe(sobjectname)
-
-        return self.field_maps[sobjectname]
+        return self.connection.get_sobject_field_map(sobjectname)
 
     def get_filtered_field_map(self, sobjectname, lam):
         field_map = self.get_field_map(sobjectname)
@@ -180,7 +161,7 @@ class Operation(object):
         return {k: field_map[k] for k in field_map if lam(field_map[k])}
 
 
-class Step(object):
+class Step(metaclass=abc.ABCMeta):
     def __init__(self, sobjectname, field_scope):
         self.sobjectname = sobjectname
         self.field_scope = field_scope
@@ -250,6 +231,7 @@ class Step(object):
             )
         }
 
+    @abc.abstractmethod
     def execute(self):
         pass
 
@@ -350,11 +332,8 @@ class LoadStep(Step):
             return value
         elif b is OutsideLookupBehavior.ERROR:
             raise AmaxaException(
-                "{} {} has an outside reference in field {} ({}), which is not allowed by the extraction configuration.",
-                self.sobjectname,
-                record_id,
-                lookup,
-                value,
+                f"{self.sobjectname} {record_id} has an outside reference in field {lookup} ({value}), "
+                "which is not allowed by the extraction configuration.",
             )
         elif b is OutsideLookupBehavior.DROP_FIELD:
             return ""
@@ -376,7 +355,7 @@ class LoadStep(Step):
                     return "false"
                 elif value.lower() in ["yes", "true", "y", "t", "1"]:
                     return "true"
-                raise ValueError("Invalid Boolean value {}", value)
+                raise ValueError(f"Invalid Boolean value {value}")
             elif value is None or len(value) == 0:
                 return None
             elif field_type == "tns:ID":
@@ -446,7 +425,7 @@ class LoadStep(Step):
                 self.context.register_error(
                     self.sobjectname,
                     original_ids[-1],
-                    "Bad data in record {}: {}".format(original_ids[-1], str(e)),
+                    f"Bad data in record {original_ids[-1]}: {str(e)}",
                 )
                 success = False
 
